# This file is part of SSHare.
#
# SSHare is free software: you can redistribute it and/or modify it under the terms of
# the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# SSHare is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# SSHare. If not, see <https://www.gnu.org/licenses/>.

class NoDefault: pass

def Flag(short=None, long=None, help=None):
    return Argument(
        short,
        long,
        action="store_const",
        const=Flag,
        default=False,
        help=help,
    )

class _NoArgument:
    def __init__(self, default):
        self._default = default

class Argument:
    def __init__(self, short=None, long=None, **kwargs):
        class _None:
            def __init__(self, default):
                self.default = default
        self._None = _None

        self.short = short
        self.long  = long

        if not "default" in kwargs:
            kwargs["default"] = NoDefault
        kwargs["default"] = _None(kwargs["default"])
        self._kwargs = kwargs

    def bind(self, plugin, argument):
        self._plugin   = plugin.name
        self._argument = argument

    def default(self):
        value = self._kwargs["default"]
        if isinstance(value, self._None):
            value = value.default
        return value

    def dest(self):
        return f"{self._plugin}_{self._argument}"

    def extract(self, arguments):
        value = getattr(arguments, self.dest())
        was_set = True
        if isinstance(value, self._None):
            was_set = False
            value = value.default
        return was_set, value

    def add(self, arg_parser):
        flags = []
        if self.short:
            flags.append(f"-{self.short}")
        long = self.long or self._argument
        if long:
            flags.append(f"--{long}")
        kwargs = self._kwargs | {
            "metavar": self._argument,
            "dest": self.dest()
        }
        arg_parser.add_argument(
            *flags,
            **kwargs
        )
